(function() {
    /*
        NAME: ReGittr
        AUTHOR: Taylar Keast
        DATE: 22/11/2017
    */

    //************
    //SETTINGS
    //************
    var reOptions = {
        //Enter github username and repo name below to complete the following URL: 'https://api.github.com/repos/GITHUB_USERNAME/REPO_NAME/commits'
        githubUsername: 'laochao',        //The username of the repository's author
        repoName: 'Colors',              //The name of the repository
        titleText: 'jQueryScript Updates',     //Display text for top op the container
        useConsole: true,                 //Set true to allow regittr to use the browser's console
        includeFontAwesome: false,        //Include FontAwesome in webpage
        expandedByDefault: false,         //Default appearance: collapsed or expanded
        appearanceDelay: 200,             //Delay (ms) before regittr appears on screen
        containerStyle: 'default',        //Theme for the container: default, dark
        containerStylesExpanded: {        //CSS styles regittr container when expanded
            'width': '20vw',
            'height': '40vh',
            'overflow-y': 'scroll'
        },
        containerStylesCollapsed: {                 //CSS styles regittr container when collapsed
            'width': '',
            'height': '',
            'max-width': '20vw',
            'overflow-y': 'hidden'
        }
    }
    var nodes = [];
    var isExpanded = reOptions.expandedByDefault;

    //************
    //ENTRY POINT
    //************
    init();
    loadNodeData('https://api.github.com/repos/' + reOptions.githubUsername + '/' + reOptions.repoName + '/commits');

    //Window load
    window.onload = function() {
        setTimeout(reAddContainer, reOptions.appearanceDelay); //Add ReGittr to the page
    }

    //************
    //FUNCTIONS
    //************
    function init() {
        //Check if jQuery has been included
        if(!window.jQuery) {
            reOut("Error - jQuery is required to run this project, please include it to continue.");
            return;
        } else reOut("Page has loaded and ReGittr is up and running!");

        //Append regittr styles and dependencies to head
        $("head").append('<link href="https://fonts.googleapis.com/css?family=Ubuntu" rel="stylesheet">');
        if(reOptions.includeFontAwesome) $("head").append('<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css">');
    }

    function reAddContainer() {
        //Initialise container and content
        $("body").prepend(
            '<div id="re-container" class="re-container re-container-' + reOptions.containerStyle + ' re-pointer">' +
                '<div id="re-header" class="re-header">' +
                '</div>' +
            '</div>');

        //Render data
        reRender();
    }

    //Output alert to console
    function reOut(msg) {
        if(reOptions.useConsole) console.log(msg);
    }

    //Add a new node to the array
    function reAddNode(node) {
        nodes.push(node);
        reRender();
    }

    //Use current array of nodes to render onto the page
    function reRender() {
        //Clear content  and apply header
        $("#re-container").html('<h1 class="re-header">' + reOptions.titleText + ' <span class="badge">' + nodes.length + '</span></h1>');
        $("#re-container").css(isExpanded ? reOptions.containerStylesExpanded : reOptions.containerStylesCollapsed);

        //Skip node rendering if container is collapsed
        if(!isExpanded) return;

        //Render all nodes into content div
        for(var i in nodes) {
            $("#re-container").append(
                '<div id="re-baseNode" class="' + nodes[i].style + '">' +
                    '<h1 class="re-baseNode-title">' + nodes[i].title + '</h1>'+
                    (nodes[i].date != null ? '<h1 class="re-baseNode-date"> on ' + nodes[i].date + '</h1>' : "") +
                    '<h1 class="re-baseNode-content">' + nodes[i].content + '</h1>' +
                '</div>');
        }

        //Show alert for no nodes found if node array is zero
        if(nodes.length < 1) $("#re-container").append('<span class="alert">Alert - no updates were found.');
    }

    //Load data from anywhere and create new nodes from data (example uses a github repo)
    function loadNodeData(url) {
        $.ajax({
            'url' : url,
            'type' : 'GET',
            'success' : function(data) {
                for(var i in data) {
                    reAddNode(new Node(data[i].committer.login, extractDateTime(data[i].commit.committer.date), data[i].commit.message));
                }
            }
        });
    }

    //Extract the date from a GitHub datetime string
    function extractDateTime(str) {
        var strSplit = str.split(/-|T|:/);
        return strSplit[1] + '/' + strSplit[2] + '/' + strSplit[0];
    }

    //************
    //CLASSES
    //************
    class Node {
        constructor(title, date, content) {
            this.id = Node.nextID++;
            this.title = title;
            this.date = date != "" ? date : null;
            this.content = content;
            this.style = styles.baseNode;
        }
    }
    Node.nextID = 0;

    //************
    //STYLES
    //************
    var styles = {
        baseNode : "re-baseNode"
    }

    //************
    //EVENTS
    //************
    $(window).on('load', function() {
        $(document).on("click", "#re-container", function() {
            isExpanded = !isExpanded;
            reRender();
        });
    });
})();
